#
#  $Id: gui_wxwin.py,v 1.2 2002/03/28 04:55:14 rob Exp $
#
#  Copyright 2001 Rob Tillotson <rob@pyrite.org>
#  All Rights Reserved
#
#  Permission to use, copy, modify, and distribute this software and
#  its documentation for any purpose and without fee or royalty is
#  hereby granted, provided that the above copyright notice appear in
#  all copies and that both the copyright notice and this permission
#  notice appear in supporting documentation or portions thereof,
#  including modifications, that you you make.
#
#  THE AUTHOR ROB TILLOTSON DISCLAIMS ALL WARRANTIES WITH REGARD TO
#  THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
#  AND FITNESS.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
#  SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
#  RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
#  CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
#  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE!
#
"""
"""

__version__ = '$Id: gui_wxwin.py,v 1.2 2002/03/28 04:55:14 rob Exp $'

__author__ = 'Rob Tillotson <rob@pyrite.org>'

__copyright__ = 'Copyright 2001 Rob Tillotson <rob@pyrite.org>'

import wx

import sys

import dtkmain
        
class PPPluginBrowser(wx.Dialog):
    ID_LISTBOX = 201
    ID_PROPERTY_GRID = 202
    
    def __init__(self, parent, instance=None):
        wx.Dialog.__init__(self, parent, -1, "Plugins", wx.DefaultPosition,
                          wx.Size(500,300))#, wx.SYSTEM_MENU|wx.CAPTION)
        self.instance = instance

        # layout:
        #   vertical box sizer
        #     horizontal sizer
        #       listbox control
        #       notebook sizer
        #         notebook
        #           info panel
        #             various static text boxes
        #           property panel
        #             dropdown box
        #             edit field
        #     horizontal sizer
        #       close button
        #

        top_sizer = wx.BoxSizer(wx.VERTICAL)

        main_sizer = wx.BoxSizer(wx.HORIZONTAL)
        # main window

        self.listbox = wx.ListBox(self, self.ID_LISTBOX)
        main_sizer.Add(self.listbox, 1, wx.EXPAND|wx.RIGHT, 5)

        pnames = self.instance.plugins.keys()
        pnames.sort()
        plugin = self.instance.plugins[pnames[0]]
        self.listbox.InsertItems(pnames,0)
        wx.EVT_LISTBOX(self, self.ID_LISTBOX, self.listbox_select)
        
        notebook = wx.Notebook(self, -1)
        
        # notebook info page
        info_panel = wx.Panel(notebook, -1)
        notebook.AddPage(info_panel, "Info")
        sz0 = wx.BoxSizer(wx.HORIZONTAL)
        sz = wx.FlexGridSizer(0,2,2,2)
        sz.Add(wx.StaticText(info_panel, -1, "Name:"))
        self.l_name = wx.StaticText(info_panel, -1, plugin.name)
        sz.Add(self.l_name)
        
        sz.Add(wx.StaticText(info_panel, -1, "Version:"))
        self.l_version=wx.StaticText(info_panel, -1, plugin.version)
        sz.Add(self.l_version)
        
        sz.Add(wx.StaticText(info_panel, -1, "Author:"))
        self.l_author = wx.StaticText(info_panel, -1, "%s <%s>" % (plugin.author, plugin.author_email))
        sz.Add(self.l_author)
        
        sz.Add(wx.StaticText(info_panel, -1, "Description:"))
        #self.l_description = wx.TextCtrl(self.info_panel, -1, "Plugin Description",
        #                                style = wx.TE_READONLY|wx.TE_MULTILINE)
        self.l_description = wx.StaticText(info_panel, -1, plugin.description)
        sz.Add(self.l_description)
        sz0.Add(sz, 1, wx.EXPAND|wx.ALL, 5)

        info_panel.SetAutoLayout(True)
        info_panel.SetSizer(sz0)

        main_sizer.Add(notebook, 2, wx.EXPAND)

        top_sizer.Add(main_sizer, 1, wx.EXPAND|wx.ALL, 5)

        # button box
        button_sizer = wx.BoxSizer(wx.HORIZONTAL)
        button_sizer.Add(wx.Button(self, wx.ID_CANCEL, "Close"), 0, wx.ALIGN_RIGHT)
            
        top_sizer.Add(button_sizer, 0, wx.ALIGN_RIGHT|wx.ALL, 5)

        self.SetAutoLayout(True)
        self.SetSizer(top_sizer)
        top_sizer.SetMinSize(wx.Size(500,300))
        top_sizer.Fit(self)
        
    def listbox_select(self, event):
        p = self.listbox.GetStringSelection()
        plugin = self.instance.plugins[p]

        self.l_name.SetLabel(plugin.name)
        self.l_version.SetLabel(plugin.version)
        self.l_author.SetLabel("%s <%s>" % (plugin.author, plugin.author_email))
        self.l_description.SetLabel(plugin.description)
        
        
class PPMainFrame(wx.Frame):
    ID_INFILENAME = 201
    ID_INFILEBUTTON = 202
    ID_GOBUTTON = 203
    ID_OUTFILEBUTTON = 204
    ID_OUTFILENAME = 205
    ID_PICK_TASK = 206
    
    M_ID_ABOUT = 101
    M_ID_EXIT = 102
    M_ID_PLUGINS = 103

    def __init__(self, parent, ID, title, instance=None):
        wx.Frame.__init__(self, parent, ID, title, wx.DefaultPosition,
                         wx.Size(500,300))

        self.instance = instance
        
        self.CreateStatusBar()

        help_menu = wx.Menu()
        help_menu.Append(self.M_ID_ABOUT, "&About",
                    "More information about Pyrite Publisher")
        wx.EVT_MENU(self, self.M_ID_ABOUT, self.menu_about)

        file_menu = wx.Menu()
        file_menu.Append(self.M_ID_PLUGINS, "&Plugins...",
                         "Plugin information and properties")
        wx.EVT_MENU(self, self.M_ID_PLUGINS, self.menu_plugins)
        file_menu.AppendSeparator()
        file_menu.Append(self.M_ID_EXIT, "E&xit",
                         "Exit Pyrite Publisher")
        wx.EVT_MENU(self, self.M_ID_EXIT, self.menu_exit)

        menuBar = wx.MenuBar()
        menuBar.Append(file_menu, "&File")
        menuBar.Append(help_menu, "&Help")

        self.SetMenuBar(menuBar)

        mainsizer = wx.BoxSizer(wx.VERTICAL)

        sz = wx.FlexGridSizer(0,3,5,5)
        sz.AddGrowableCol(1) # where is this documented???
        
        sz.Add(wx.StaticText(self, -1, "Input File:"), 0, wx.ALL|wx.ALIGN_RIGHT, 5)
        self.ctrl_infile = wx.TextCtrl(self, self.ID_INFILENAME)
        sz.Add(self.ctrl_infile, 1, wx.EXPAND)
        sz.Add(wx.Button(self, self.ID_INFILEBUTTON, "Choose...", wx.DefaultPosition),0)
        wx.EVT_BUTTON(self, self.ID_INFILEBUTTON, self.button_chooseinput)

        sz.Add(wx.StaticText(self, -1, "Output File:"), 0, wx.ALL|wx.ALIGN_RIGHT, 5)
        self.ctrl_outfile = wx.TextCtrl(self, self.ID_OUTFILENAME)
        sz.Add(self.ctrl_outfile, 1, wx.EXPAND)
        sz.Add(wx.Button(self, self.ID_OUTFILEBUTTON, "Choose...", wx.DefaultPosition),0)
        wx.EVT_BUTTON(self, self.ID_OUTFILEBUTTON, self.button_chooseoutput)

        # task combobox
        sz.Add(wx.StaticText(self, -1, "Task:"), 0, wx.ALL|wx.ALIGN_RIGHT, 5)
        self.tasks = self.instance.tasks.items()
        self.tasks.sort()
        self.pick_task = wx.Choice(self, self.ID_PICK_TASK, wx.DefaultPosition,
                                  wx.DefaultSize, ["Automatic conversion"]+\
                                  [t.title for k,t in self.tasks])
        self.pick_task.SetSelection(0)
        sz.Add(self.pick_task, 1, wx.EXPAND)
        sz.Add(wx.StaticText(self, -1, ""), 0, wx.ALL|wx.ALIGN_RIGHT, 5)
        
        mainsizer.Add(sz, 0, wx.EXPAND|wx.ALL, 5)

        
        sz0 = self.init_advanced_options()
        mainsizer.Add(sz0,2,wx.ALL|wx.EXPAND,5)
        
        # button box
        sz = wx.BoxSizer(wx.HORIZONTAL)
        sz.Add(wx.Button(self, self.ID_GOBUTTON, "Convert"), 0)
        wx.EVT_BUTTON(self, self.ID_GOBUTTON, self.button_convert)
        mainsizer.Add(sz,0,wx.ALL|wx.ALIGN_RIGHT,5)

        self.SetAutoLayout(True)
        self.SetSizer(mainsizer)
        
    #
    # ADVANCED OPTIONS
    # A lot of these are more restricted versions of what the plugins can do.
    # Oh well, such is life when you insist on a GUI instead of using the
    # shell like a real man :)
    #
    ID_CHECK_INSTALL = 300
    ID_PICK_DOC_OUTPUT = 301
    ID_PICK_TEXT_PARSER = 302
    ID_CHECK_COPYDOC = 303
    ID_PICK_DOC_ASSEMBLER = 304
    
    def init_advanced_options(self):
        # advanced options
        opts = wx.StaticBox(self, -1, "Advanced Options")
        sz0 = wx.StaticBoxSizer(opts, wx.VERTICAL)

        sz = wx.FlexGridSizer(0,2,5,5)
        sz.AddGrowableCol(0)
        sz.AddGrowableCol(1)

        # ** Queue for installation ; sets the "install" property on every plugin with it
        # and disables the output file box if present
        if self.instance.find_installer() is not None:
            self.check_install = wx.CheckBox(self, self.ID_CHECK_INSTALL, "Queue for installation")
            sz.Add(self.check_install)
        else:
            self.check_install = None
        #wx.EVT_CHECKBOX(self, self.ID_CHECK_INSTALL, self.toggle_install)

        # ** Default doc output method: looks for plugins with doc-output
        # and lets you pick one, and forces it
        #z = wx.BoxSizer(wx.HORIZONTAL)
        #pnames = [ p.name for p in self.instance.find_plugins_with_link('doc-output') ]
        #pnames.insert(0,"<Automatic>")
        #z.Add(wx.StaticText(self, -1, "Doc Output:"), 0, wx.ALIGN_CENTER_VERTICAL)
        #self.pick_docoutput = wx.Choice(self, self.ID_PICK_DOC_OUTPUT, wx.DefaultPosition,
        #                               wx.DefaultSize,
        #                               pnames)
        #z.Add(self.pick_docoutput, 1)
        #self.pick_docoutput.SetSelection(0)
        #sz.Add(z, 1, wx.EXPAND)

        # ** Copy docs directly: Forces the CopyDoc plugin
        self.check_copydoc = wx.CheckBox(self, self.ID_CHECK_COPYDOC, "Copy Docs directly")
        sz.Add(self.check_copydoc)

        # ** Doc format: lets you pick a doc-assembler
        #z = wx.BoxSizer(wx.HORIZONTAL)
        #pnames = [ p.name for p in self.instance.find_plugins_with_link('doc-assembler') ]
        #pnames.insert(0, "<Automatic>")
        #z.Add(wx.StaticText(self, -1, "Doc Format:"), 0, wx.ALIGN_CENTER_VERTICAL)
        #self.pick_docassembler = wx.Choice(self, self.ID_PICK_DOC_ASSEMBLER, wx.DefaultPosition,
        #                                  wx.DefaultSize,
        #                                  pnames)
        #z.Add(self.pick_docassembler, 1)
        #self.pick_docassembler.SetSelection(0)
        #sz.Add(z, 1, wx.EXPAND)

        sz.Add(wx.StaticText(self, -1, "")) # empty
        
        # ** Default text conversion method: looks for plugins with text/plain and
        # lets you pick one
        #z = wx.BoxSizer(wx.HORIZONTAL)
        #pnames = [ p.name for p in self.instance.find_plugins_with_link('text/plain') ]
        #pnames.insert(0, "<Automatic>")
        #z.Add(wx.StaticText(self, -1, "Text Parser:"), 0, wx.ALIGN_CENTER_VERTICAL)
        #self.pick_textparser = wx.Choice(self, self.ID_PICK_TEXT_PARSER, wx.DefaultPosition,
        #                                wx.DefaultSize,
        #                                pnames)
        #z.Add(self.pick_textparser, 1)
        #self.pick_textparser.SetSelection(0)
        #sz.Add(z, 1, wx.EXPAND)

        sz0.Add(sz,1,wx.EXPAND)
        return sz0


    def do_options(self):
        # properties
        if self.check_install is not None and self.check_install.GetValue():
            self.instance.set_property_all('install',1)
        else:
            self.instance.set_property_all('install',0)
        self.instance.set_property_all('output_filename',self.ctrl_outfile.GetValue())

        # task (if any)
        n = self.pick_task.GetSelection()
        print n
        if n:
            k,task = self.tasks[n-1]
            forced_plugins = task.plugins[:]
            for k,v in task.properties.items():
                print "Task property:", k, "=", v
                self.set_property_all(k,v)
        else:
            forced_plugins = []
            
        # forced plugins
        if self.check_copydoc.GetValue():
            forced_plugins.append('CopyDoc')
        #p.append(self.pick_docoutput.GetStringSelection())
        #p.append(self.pick_docassembler.GetStringSelection())
        #p.append(self.pick_textparser.GetStringSelection())

        fp = [self.instance.plugins[x] for x in forced_plugins]
        return fp
    
    #################################################
    def menu_exit(self, event):
        self.Close(True)

    def menu_about(self, event):
        dlg = wx.MessageDialog(self, "Pyrite Publisher %s\nby Rob Tillotson <rob@pyrite.org>\nhttp://www.pyrite.org/" % dtkmain.VERSION,
                              "About Pyrite Publisher", wx.OK)
        dlg.ShowModal()
        dlg.Destroy()

    def menu_plugins(self, event):
        dlg = PPPluginBrowser(self, self.instance)
        dlg.Show(True)
        
    def button_chooseinput(self, event):
        wc = self.instance.wildcards.items()
        wc.sort(lambda x,y: cmp(x[1],y[1]))
        wc.insert(0, ('*.*', 'All files'))
        wct = '|'.join(map(lambda p: '%s (%s)|%s' % (p[1],p[0],p[0]), wc))
        
        dlg = wx.FileDialog(self, "Choose a file to convert",
                           wildcard = wct, style=wx.FD_OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            ifn = dlg.GetPath()
            self.ctrl_infile.SetValue(ifn)
            self.ctrl_outfile.SetValue('')
        dlg.Destroy()
        
    def button_chooseoutput(self, event):
        dlg = wx.FileDialog(self, "Choose an output filename", wildcard="All files (*.*)|*.*",
                           style=wx.FD_SAVE)
        if dlg.ShowModal() == wx.ID_OK:
            ofn = dlg.GetPath()
            self.ctrl_outfile.SetValue(ofn)
        dlg.Destroy()
        
    
    def button_convert(self, event):
        fname = self.ctrl_infile.GetValue()
        self.SetStatusText("Converting %s..." % fname)

        forced = self.do_options()
        #try:
        self.instance.convert_file(fname, forced)
        #except dtkmain.ConversionError, err:
        #    dlg = wx.MessageDialog(self, str(err), "Conversion Error",
        #                          wx.OK|wx.ICON_EXCLAMATION)
        #    dlg.ShowModal()
        #    dlg.Destroy()
        #except:
        #    t, v, tb = sys.exc_info()
        #    dlg = wx.MessageDialog(self, "%s: %s" % (t,v), "Error",
        #                          wx.OK|wx.ICON_EXCLAMATION)
        #    dlg.ShowModal()
        #    dlg.Destroy()

        self.SetStatusText("Done.")


        
class PPApp(wx.App,dtkmain.PPInstance):
    def __init__(self, *a, **kw):
        dtkmain.PPInstance.__init__(self)
        wx.App.__init__(self, *a, **kw)
        
    def OnInit(self):
        frame = PPMainFrame(None, -1, "Pyrite Publisher", instance=self)
        frame.Show(True)
        self.SetTopWindow(frame)
        return True


if __name__ == '__main__':
    app = PPApp(0)
    app.MainLoop()
    
